﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Notification; // <-=
using System.Collections.ObjectModel;

namespace PushNotifications1
{
    public partial class MainPage : PhoneApplicationPage
    {
        public HttpNotificationChannel myChannel;
        const string channelName = "NikitaP_AppChannel";
        const string serviceName = "www.nikitapolyakov.com/MyApp";

        public MainPage()
        {
            InitializeComponent();

            FindOrCreateNotificationChannel();
        }

        public void FindOrCreateNotificationChannel()
        {
            // Attempt to find my channel
            myChannel = HttpNotificationChannel.Find(channelName);
            
            // Check if the channel exists
            if (myChannel == null)
            {
                // If not, let's create the channel with our channel and service names
                myChannel = new HttpNotificationChannel(channelName, serviceName);

                // Before we open the channel let's set up event delegates
                SetUpEventDelegates();

                // After myChannel.Open() is called, the notification channel URI will be sent to the application through the ChannelUriUpdated delegate.
                // If your application requires a timeout for setting up a notification channel, start it after the myChannel.Open() call. 
                myChannel.Open();
            }
            else 
            {
                // Found an existing notification channel.
                // Since we are just initializing we need to re-setup the event delegates
                SetUpEventDelegates();

                // The URI that the application can send to its web service.
                // For testing purposes in this case we come back to the UI thread to update a text box with our URI
                this.Dispatcher.BeginInvoke(() =>
                    {
                        // To simply copy&paste to our text application simply place a breakpoint in the next line to intercept the value in Debug mode
                        ChannelURITextBox.Text = myChannel.ChannelUri.ToString();
                    }
                );
            }
        }

        /// <summary>
        /// Sets up the event delegates for the events that happen on myChannel
        /// </summary>
        public void SetUpEventDelegates()
        {
            myChannel.ChannelUriUpdated += new EventHandler<NotificationChannelUriEventArgs>(myChannel_ChannelUriUpdated);
            myChannel.HttpNotificationReceived += new EventHandler<HttpNotificationEventArgs>(myChannel_HttpNotificationReceived);
        }

        void myChannel_ChannelUriUpdated(object sender, NotificationChannelUriEventArgs e)
        {
            // In most cases this is where you would send your URI to you webservices, e.ChannelUri contains the URI
            // For testing purposes in this case we come back to the UI thread to update a text box with our URI            
            this.Dispatcher.BeginInvoke(() =>
                {
                    // To simply copy&paste to our text application simply place a breakpoint in the next line to intercept the value in Debug mode
                    ChannelURITextBox.Text = e.ChannelUri.ToString();
                }
            );
        }

        // Handeling a Raw Notification
        void myChannel_HttpNotificationReceived(object sender, HttpNotificationEventArgs e)
        {
            // Check that we got the message fully
            if (e.Notification.Body != null && e.Notification.Headers != null)
            {
                // Using a steam reader read in the contents of the Raw message
                System.IO.StreamReader reader = new System.IO.StreamReader(e.Notification.Body);
                
                // Conver the message from byte array back to a String
                string messageText = reader.ReadToEnd().ToString();
                
                // Back on the UI thread set the Output TextBlock to the message
                this.Dispatcher.BeginInvoke(() =>
                    {
                        Output.Text = "Raw Notification says: " + messageText;
                    }
                 );   
            }
        }
    }
}