﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;
using System.IO;

namespace NotificationService
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
        }

        private void SendRaw_Click(object sender, RoutedEventArgs e)
        {
            // Using the URI Channel input from the screen, create a HTTP request
            HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(URI.Text);
            
            // Set up the properties of the request
            sendNotificationRequest.Method = "POST";
            sendNotificationRequest.Headers = new WebHeaderCollection();
            sendNotificationRequest.ContentType = "text/xml";

            sendNotificationRequest.Headers.Add("X-NotificationClass", "3");    // Raw Notification type for Immediate delivery 
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "13");   // Raw Notification type for delivery within 450 seconds
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "23");   // Raw Notification type for delivery within 900 seconds
            
            // Define the payload to be sent, we are taking content of the TextBox on the screen and encoding it into a byte array
            byte[] contentBytes = new UTF8Encoding().GetBytes(RawContent.Text);

            // Setup the content length and fill the steam with the message bytes
            sendNotificationRequest.ContentLength = contentBytes.Length;            
            using (Stream requestStream = sendNotificationRequest.GetRequestStream())
            {
                requestStream.Write(contentBytes, 0, contentBytes.Length);
            }

            // Attempt to send the response
            try
            {
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();

                string notificationStatus       = response.Headers["X-NotificationStatus"];
                string subscriptionStatus       = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus   = response.Headers["X-DeviceConnectionStatus"];

                // Write the status update to the Status TextBlock in the UI
                Status.Text = "Raw Status: " + notificationStatus + " | " + deviceConnectionStatus + " | " + subscriptionStatus;
            }
            catch (Exception ex)
            {
                // Output the error message in the status
                Status.Text = "Raw Error: " + ex.Message;
            }
        }

        string ToastPushXML =
            "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
            "<wp:Notification xmlns:wp=\"WPNotification\">" +
               "<wp:Toast>" +
                  "<wp:Text1>{0}</wp:Text1>" +
                  "<wp:Text2>{1}</wp:Text2>" +
               "</wp:Toast>" +
            "</wp:Notification>";

        private void SendToast_Click(object sender, RoutedEventArgs e)
        {
            // Using the URI Channel input from the screen, create a HTTP request
            HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(URI.Text);

            // Set up the properties of the request
            sendNotificationRequest.Method = "POST";
            sendNotificationRequest.Headers = new WebHeaderCollection();
            sendNotificationRequest.ContentType = "text/xml";

            sendNotificationRequest.Headers.Add("X-WindowsPhone-Target", "toast");
            sendNotificationRequest.Headers.Add("X-NotificationClass", "2");        // Toast Notification type for Immediate delivery 
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "12");    // Toast Notification type for delivery within 450 seconds
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "22");    // Toast Notification type for delivery within 900 seconds

            // Fill in the TextBox values in the XML string we defined earlier
            string str = string.Format(ToastPushXML, ToastTitle.Text, ToastText.Text);

            // Define the payload to be sent, by enrypting the resulting XML string from above
            byte[] strBytes = new UTF8Encoding().GetBytes(str);


            // Setup the content length and fill the steam with the message bytes
            sendNotificationRequest.ContentLength = strBytes.Length;
            using (Stream requestStream = sendNotificationRequest.GetRequestStream())
            {
                requestStream.Write(strBytes, 0, strBytes.Length);
            }


            // Attempt to send the response
            try
            {
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();

                string notificationStatus = response.Headers["X-NotificationStatus"];
                string subscriptionStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];

                // Write the status update to the Status TextBlock in the UI
                Status.Text = "Toast Status: " + notificationStatus + " | " + deviceConnectionStatus + " | " + subscriptionStatus;
            }
            catch (Exception ex)
            {
                Status.Text = "Toast Error: " + ex.Message;
            }
        }

        string TilePushXML =
            "<?xml version=\"1.0\" encoding=\"utf-8\"?>" +
            "<wp:Notification xmlns:wp=\"WPNotification\">" +
               "<wp:Tile>" +
                  "<wp:BackgroundImage>{2}</wp:BackgroundImage>" +
                  "<wp:Count>{1}</wp:Count>" +
                  "<wp:Title>{0}</wp:Title>" +
               "</wp:Tile> " +
            "</wp:Notification>";

        private void SendLiveTile_Click(object sender, RoutedEventArgs e)
        {
            // Using the URI Channel input from the screen, create a HTTP request
            HttpWebRequest sendNotificationRequest = (HttpWebRequest)WebRequest.Create(URI.Text);

            // Set up the properties of the request
            sendNotificationRequest.Method = "POST";
            sendNotificationRequest.Headers = new WebHeaderCollection();
            sendNotificationRequest.ContentType = "text/xml";

            sendNotificationRequest.Headers.Add("X-WindowsPhone-Target", "token");
            sendNotificationRequest.Headers.Add("X-NotificationClass", "1");        // Tile Notification type for Immediate delivery 
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "11");    // Tile Notification type for delivery within 450 seconds
            // sendNotificationRequest.Headers.Add("X-NotificationClass", "21");    // Tile Notification type for delivery within 900 seconds

            // Fill in the TextBox values in the XML string we defined earlier
            string str = string.Format(TilePushXML, TileNumber.Text, TileTitle.Text, TileBackground.Text);

            // Define the payload to be sent, by enrypting the resulting XML string from above
            byte[] strBytes = new UTF8Encoding().GetBytes(str);

            // Setup the content length and fill the steam with the message bytes
            sendNotificationRequest.ContentLength = strBytes.Length;
            using (Stream requestStream = sendNotificationRequest.GetRequestStream())
            {
                requestStream.Write(strBytes, 0, strBytes.Length);
            }

            // Attempt to send the response
            try
            {
                HttpWebResponse response = (HttpWebResponse)sendNotificationRequest.GetResponse();

                string notificationStatus = response.Headers["X-NotificationStatus"];
                string subscriptionStatus = response.Headers["X-SubscriptionStatus"];
                string deviceConnectionStatus = response.Headers["X-DeviceConnectionStatus"];

                // Write the status update to the Status TextBlock in the UI
                Status.Text = "Tile Status: " + notificationStatus + " | " + deviceConnectionStatus + " | " + subscriptionStatus;
            }
            catch (Exception ex)
            {
                Status.Text = "Tile Error: " + ex.Message;
            }
        }
    }
}
