using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using Microsoft.Devices.Sensors;

namespace Chapter18Game
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Sphere gameBall;
        Sphere goal;
        Sphere hazardArea;

        Accelerometer accelerometer;
        Vector3 accelerometerReading = new Vector3();

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Extend battery life under lock.
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            graphics.SupportedOrientations = DisplayOrientation.LandscapeLeft;
        }

        protected override void BeginRun()
        {
            base.BeginRun();

            accelerometer.Start();
        }

        protected override void EndRun()
        {
            base.EndRun();

            accelerometer.Stop();
        }

        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            gameBall = new Sphere()
            {
                Texture = Content.Load<Texture2D>("Ball"),
                Position = new Vector2(400, 30)
            };

            goal = new Sphere()
            {
                Texture = Content.Load<Texture2D>("Goal"),
                Position = new Vector2(240, 240)
            };

            hazardArea = new Sphere()
            {
                Texture = Content.Load<Texture2D>("HazardArea"),
                Position = new Vector2(560, 240)
            };



            gameBall.Magnitude = 2;
            gameBall.Direction = 180;
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            accelerometer = new Accelerometer();
            accelerometer.ReadingChanged += (sender, args) =>
            {
                accelerometerReading.Y = -(float)args.X * 2f;
                accelerometerReading.X = -(float)args.Y * 2f;
                accelerometerReading.Z = (float)args.Z;
            };
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {

        }

        void CheckForBorderCollision()
        {
            if (this.gameBall.BoundingBox.Left <= 0)
            {
                this.gameBall.Force = new Vector2(Math.Abs(this.gameBall.Force.X),
                   this.gameBall.Force.Y);
            }

            if (this.gameBall.BoundingBox.Top <= 0)
            {
                this.gameBall.Force = new Vector2(gameBall.Force.X,
                   Math.Abs(this.gameBall.Force.Y));
            }

            if (this.gameBall.BoundingBox.Right >= ScreenSize.X)
            {
                this.gameBall.Force = new Vector2(-Math.Abs(this.gameBall.Force.X),
                   this.gameBall.Force.Y);
            }

            if (this.gameBall.BoundingBox.Bottom >= ScreenSize.Y)
            {
                this.gameBall.Force = new Vector2(gameBall.Force.X,
                   -Math.Abs(this.gameBall.Force.Y));
            }
        }


        void CheckForGoalCollision()
        {
            if (this.goal.Contains(this.gameBall))
            {
                System.Diagnostics.Debugger.Break();
            }
        }


        void CheckForHazardAreaCollision()
        {
            if (this.gameBall.Collided(this.hazardArea))
            {
                System.Diagnostics.Debugger.Break();
            }
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            this.gameBall.Force = new Vector2(this.gameBall.Force.X +
               this.accelerometerReading.X, this.gameBall.Force.Y +
               this.accelerometerReading.Y);

            CheckForGoalCollision();
            CheckForBorderCollision();
            CheckForHazardAreaCollision();

            gameBall.Update(gameTime.ElapsedGameTime);
            base.Update(gameTime);
        }

        public Vector2 ScreenSize
        {
            get
            {
                return new Vector2(graphics.GraphicsDevice.Viewport.Width,
                                   graphics.GraphicsDevice.Viewport.Height);
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            spriteBatch.Draw(goal.Texture, goal.RenderPosition, Color.White);
            spriteBatch.Draw(hazardArea.Texture, hazardArea.RenderPosition, Color.White);
            spriteBatch.Draw(gameBall.Texture, gameBall.RenderPosition, Color.White);
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
