using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework;

namespace Chapter18Game
{
    public class Sphere
    {
        private const double TargetInterval = 33.3;

        public Texture2D Texture
        {
            get;
            set;
        }

        public float Magnitude
        {
            get;
            set;
        }

        public float Direction
        {
            get;
            set;
        }

        public Vector2 Force
        {
            get
            {
                double radians = MathHelper.ToRadians(Convert.ToSingle(this.Direction));
                float xDelta = Convert.ToSingle(Math.Sin(radians) * this.Magnitude);
                float yDelta = -Convert.ToSingle(Math.Cos(radians) * this.Magnitude);

                return new Vector2(xDelta, yDelta);
            }
            set
            {
                double atanResult = Math.Atan2(value.X, -value.Y);
                float dir = MathHelper.ToDegrees(Convert.ToSingle(atanResult));
                this.Direction = (dir < 0) ? 360 + dir : dir;
                this.Magnitude = Convert.ToSingle(
                   Math.Sqrt(value.X * value.X + value.Y * value.Y));
            }
        }

        public Vector2 Position
        {
            get;
            set;
        }

        public Vector2 RenderPosition
        {
            get
            {
                Vector2 value = this.Position;
                if (this.Texture != null)
                {
                    value = Position - new Vector2(Texture.Width / 2, Texture.Height / 2);
                }

                return value;
            }
        }

        public float Radius
        {
            get
            {
                var value = 0f;
                if (this.Texture != null)
                {
                    value = this.Texture.Width / 2;
                }

                return value;
            }
        }

        public Rectangle BoundingBox
        {
            get
            {
                var rectangle = Rectangle.Empty;
                if (this.Texture != null)
                {
                    rectangle = new Rectangle(
                    Convert.ToInt32(this.RenderPosition.X),
                    Convert.ToInt32(this.RenderPosition.Y),
                    this.Texture.Width,
                    this.Texture.Height);
                }

                return rectangle;
            }
        }

        public bool Collided(Sphere sphere2)
        {
            var threshold = this.Radius + sphere2.Radius;
            return (sphere2.Position - this.Position).Length() <= threshold;
        }

        public bool Contains(Sphere sphere2)
        {
            var delta = (this.Position - sphere2.Position).Length();
            return delta <= this.Radius;
        }


        private Vector2 GetDelta(TimeSpan elapsedTime)
        {
            var numberIntervals = elapsedTime.TotalMilliseconds / Sphere.TargetInterval;
            return this.Force * Convert.ToSingle(numberIntervals);
        }

        public void Update(TimeSpan elapsedTime)
        {
            var delta = this.GetDelta(elapsedTime);
            this.Position += delta;
        }
    }
}
