using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using Microsoft.Devices.Sensors;
using Microsoft.Devices;

namespace Chapter19Game
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        Sphere gameBall;
        Sphere goal;
        Sphere hazardArea;

        Accelerometer accelerometer;
        Vector3 accelerometerReading = new Vector3();

        GameState currentGameState;
        SpriteFont statusFont;
        SpriteFont titleFont;

        SoundEffect boom;
        SoundEffect bounce;
        SoundEffect buzzer;
        SoundEffect drop;

        int score;
        int numberOfBalls;

        enum GameState
        {
            Start,
            Active,
            BallDropped,
            BallDropping,
            HitHazardArea,
            Done
        }

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            // Extend battery life under lock.
            InactiveSleepTime = TimeSpan.FromSeconds(1);

            graphics.SupportedOrientations = DisplayOrientation.LandscapeLeft;
        }

        protected override void BeginRun()
        {
            base.BeginRun();

            accelerometer.Start();
        }

        protected override void EndRun()
        {
            base.EndRun();

            accelerometer.Stop();
        }

        void StartGame()
        {
            this.numberOfBalls = 5;
            this.score = 0;
            this.gameBall.Position = new Vector2(30, 30);
            this.gameBall.Force = Vector2.Zero;
            this.currentGameState = GameState.Active;
        }


        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            gameBall = new Sphere()
            {
                Texture = Content.Load<Texture2D>("Ball"),
                Position = new Vector2(400, 30)
            };

            goal = new Sphere()
            {
                Texture = Content.Load<Texture2D>("Goal"),
                Position = new Vector2(240, 240)
            };

            hazardArea = new Sphere()
            {
                Texture = Content.Load<Texture2D>("HazardArea"),
                Position = new Vector2(560, 240)
            };

            this.statusFont = Content.Load<SpriteFont>("Status");
            this.titleFont = Content.Load<SpriteFont>("Title");

            gameBall.Magnitude = 2;
            gameBall.Direction = 180;

            this.bounce = Content.Load<SoundEffect>("Bounce");
            this.boom = Content.Load<SoundEffect>("Boom");
            this.buzzer = Content.Load<SoundEffect>("Buzzer");
            this.drop = Content.Load<SoundEffect>("Drop");
        }


        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            base.Initialize();

            accelerometer = new Accelerometer();
            accelerometer.ReadingChanged += (sender, args) =>
            {
                accelerometerReading.Y = -(float)args.X * 2f;
                accelerometerReading.X = -(float)args.Y * 2f;
                accelerometerReading.Z = (float)args.Z;
            };
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {

        }

        void CheckForBorderCollision()
        {
            if (this.gameBall.BoundingBox.Left <= 0)
            {
                this.gameBall.Force = new Vector2(Math.Abs(this.gameBall.Force.X),
                   this.gameBall.Force.Y);
                Vibrate(50);
                this.bounce.Play();
            }

            if (this.gameBall.BoundingBox.Top <= 0)
            {
                this.gameBall.Force = new Vector2(gameBall.Force.X,
                   Math.Abs(this.gameBall.Force.Y));
                Vibrate(50);
                this.bounce.Play();                
            }

            if (this.gameBall.BoundingBox.Right >= ScreenSize.X)
            {
                this.gameBall.Force = new Vector2(-Math.Abs(this.gameBall.Force.X),
                   this.gameBall.Force.Y);
                Vibrate(50);
                this.bounce.Play();
            }

            if (this.gameBall.BoundingBox.Bottom >= ScreenSize.Y)
            {
                this.gameBall.Force = new Vector2(gameBall.Force.X,
                   -Math.Abs(this.gameBall.Force.Y));
                Vibrate(50);
                this.bounce.Play();
            }
        }


        void CheckForGoalCollision()
        {
            if (this.goal.Contains(this.gameBall))
            {
                if (this.goal.Contains(this.gameBall))
                    this.currentGameState = GameState.BallDropping;
            }
        }


        void CheckForHazardAreaCollision()
        {
            if (this.gameBall.Collided(this.hazardArea))
            {
                --this.numberOfBalls;

                if (this.numberOfBalls == 0)
                {
                    this.currentGameState = GameState.Done;
                    this.buzzer.Play();
                }
                else
                {
                    this.currentGameState = GameState.HitHazardArea;
                    this.boom.Play();
                }
            }
        }

        void Vibrate(int ms)
        {
            VibrateController.Default.Start(TimeSpan.FromMilliseconds(ms));
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            var screenTapped = TouchPanel.GetState().Count > 0;

            switch (currentGameState)
            {
                case GameState.Done:
                case GameState.Start:
                    if (screenTapped)
                    {
                        StartGame();
                    }

                    break;

                case GameState.Active:
                    this.gameBall.Update(gameTime.ElapsedGameTime);
                    this.gameBall.Force = new Vector2(
                       this.gameBall.Force.X + this.accelerometerReading.X,
                       this.gameBall.Force.Y + this.accelerometerReading.Y);

                    CheckForBorderCollision();
                    CheckForGoalCollision();
                    CheckForHazardAreaCollision();
                    break;

                case GameState.HitHazardArea:
                case GameState.BallDropped:
                    if (screenTapped)
                    {
                        this.currentGameState = GameState.Active;
                        this.gameBall.Position = new Vector2(30, 30);
                        this.gameBall.Force = Vector2.Zero;
                    }

                    break;

                case GameState.BallDropping:
                    var delta = goal.Position - gameBall.Position;
                    this.gameBall.Force = delta / 5.0f;
                    this.gameBall.Update(gameTime.ElapsedGameTime);

                    if (delta.Length() < 1)
                    {
                        this.score += 20;
                        this.currentGameState = GameState.BallDropped;
                        this.drop.Play();
                    }

                    break;

                default:
                    this.gameBall.Update(gameTime.ElapsedGameTime);
                    this.gameBall.Force = new Vector2(
                       this.gameBall.Force.X + this.accelerometerReading.X,
                       this.gameBall.Force.Y + this.accelerometerReading.Y);

                    CheckForBorderCollision();
                    CheckForGoalCollision();
                    CheckForHazardAreaCollision();
                    break;
            }

            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            {
                this.Exit();
            }

            base.Update(gameTime);

        }

        public Vector2 ScreenSize
        {
            get
            {
                return new Vector2(graphics.GraphicsDevice.Viewport.Width,
                                   graphics.GraphicsDevice.Viewport.Height);
            }
        }

        void DrawTitleMessage(string message)
        {
            var size = this.titleFont.MeasureString(message);
            var left = (ScreenSize.X / 2) - (size.X / 2);
            var top = (ScreenSize.Y / 2) - (size.Y / 2);
            spriteBatch.DrawString(this.titleFont, message, new Vector2(left, top),
               Color.Yellow);
        }


        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            spriteBatch.Begin();
            switch (this.currentGameState)
            {
                case GameState.Start:
                    DrawTitleMessage("Tap the screen to start!");
                    break;

                case GameState.Active:
                    var scoreMessage = String.Format("Score: {0} Number Balls: {1}",
                       this.score, this.numberOfBalls);
                    spriteBatch.DrawString(this.statusFont, scoreMessage, new Vector2(5, 5),
                       Color.White);

                    spriteBatch.Draw(goal.Texture, goal.RenderPosition, Color.White);
                    spriteBatch.Draw(hazardArea.Texture, hazardArea.RenderPosition, Color.White);
                    spriteBatch.Draw(gameBall.Texture, gameBall.RenderPosition, Color.White);
                    break;

                case GameState.HitHazardArea:
                case GameState.BallDropped:
                    spriteBatch.Draw(goal.Texture, goal.RenderPosition, Color.White);
                    spriteBatch.Draw(hazardArea.Texture, hazardArea.RenderPosition, Color.White);
                    spriteBatch.Draw(gameBall.Texture, gameBall.RenderPosition, Color.White);
                    DrawTitleMessage("Tap the screen to continue!");
                    break;

                case GameState.Done: DrawTitleMessage(
                   String.Format("Score: {0}\r\nGame Over\r\nTap the screen to Restart!", score));
                    break;


                default:
                    spriteBatch.Draw(goal.Texture, goal.RenderPosition, Color.White);
                    spriteBatch.Draw(hazardArea.Texture, hazardArea.RenderPosition,
                       Color.White);
                    spriteBatch.Draw(gameBall.Texture, gameBall.RenderPosition, Color.White);
                    break;
            }
            spriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
