﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;

using BingSearchSDK;
using System.IO;
using System.Xml;
using System.Xml.Linq;
using System.Text;

namespace BingSearchSDK
{
    public partial class NewsXMLPage : PhoneApplicationPage
    {
        OutMsg m_Out;
        String m_AppId = string.Empty;

        // Constructor
        public NewsXMLPage()
        {
            InitializeComponent();
            // m_Out = new OutMsg(textBlockOut);
            m_Out = new OutMsg();
            m_AppId = Application.Current.Resources["AppId"] as String;
        }

        private void buttonSearch_Click(object sender, RoutedEventArgs e)
        {
            m_Out.Clear();
            m_Out.WriteLine("XML API");
            HttpWebRequest req = BuildXMLRequest();
            try
            {
                req.BeginGetResponse(new AsyncCallback(ReadWebRequestCallback), req);
            }
            catch (Exception ex)
            {
                m_Out.WriteLine(ex.ToString());
            }
        } // search

        private HttpWebRequest BuildXMLRequest()
        {
            string requestString = "http://api.bing.net/xml.aspx?"

                // Common request fields (required)
                + "AppId=" + m_AppId
                + "&Query=" + textBoxSearch.Text
                + "&Sources=News"

                // Common request fields (optional)
                + "&Market=en-us";

            // Create and initialize the request.
            HttpWebRequest request = (HttpWebRequest)HttpWebRequest.Create(
                requestString);

            return request;
        }

        private void ReadWebRequestCallback(IAsyncResult callbackResult)
        {
            HttpWebRequest myRequest = (HttpWebRequest)callbackResult.AsyncState;
            HttpWebResponse myResponse = (HttpWebResponse)myRequest.EndGetResponse(callbackResult);

            using (StreamReader httpwebStreamReader = new StreamReader(myResponse.GetResponseStream()))
            {
                string results = httpwebStreamReader.ReadToEnd();
                // Dispatcher.BeginInvoke(() => m_Out.WriteLine(results.Substring(0,300)));
                DisplayResponse(results);
            }
        }

        private void DisplayResponse(string result)
        {
            Dispatcher.BeginInvoke(() => m_Out.WriteLine("result", result ));

            System.Xml.Linq.XDocument _xdoc = System.Xml.Linq.XDocument.Parse( result );

            XNamespace news = "http://schemas.microsoft.com/LiveSearch/2008/04/XML/news";
            
            // get a relative root
            var searchItems = _xdoc.Descendants(news + "Results");
            
            Dispatcher.BeginInvoke(() => m_Out.WriteLine("searchItems", searchItems.Count().ToString()));

            //foreach (XElement myElem in searchItems)
            //{
            //    Dispatcher.BeginInvoke(() =>
            //        m_Out.WriteLine(myElem.Value + "")
            //        );
            //}

            // var webresults = from _result in searchItems.Descendants(web + "WebResult") select _result;

            //List<BingWebResult> _results =
            //    (from _result in searchItems.Descendants(web + "WebResult") 
            //    select new BingWebResult
            //    {
            //        Title = _result.Element(web + "Title").Value
            //    }).ToList();

            IEnumerable<NewsArticle> _results =
            from _result in searchItems.Descendants(news + "NewsResult")
            select new NewsArticle
            {
                Title = _result.Element(news + "Title").Value,
                Source = _result.Element(news + "Source").Value,
                Url = _result.Element(news + "Url").Value,
                Snippet = _result.Element(news + "Snippet").Value,
                Date = _result.Element(news + "Date").Value
            };

            Dispatcher.BeginInvoke(() =>
                listBoxTarget.ItemsSource = _results
            );

            // puke city - only dumps last element - some kind of dispatcher latency - jeh
            //foreach (var w in _results)
            //{
            //    Dispatcher.BeginInvoke(() =>
            //        m_Out.WriteLine(
            //            String.Format("{0}", w.Title)
            //            ));
            //}


            StringBuilder sb = new StringBuilder(1024);

            // ugly but helped me debug
            //for (int ii = 0; ii < (_results.Count-1); ii++)
            //{
            //    sb.AppendLine(_results[ii].Title);
            //}

            foreach (var w in _results)
            {
                sb.AppendFormat("Date: {0}", w.Date);
                sb.AppendLine(w.Title);
            }

            Dispatcher.BeginInvoke(() =>
                m_Out.WriteLine(
                String.Format(sb.ToString())
            ));

        } // displayresponse
    }
}