﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Microsoft.Phone.Controls;
using System.IO;

using System.IO.IsolatedStorage;
using Microsoft.Xna.Framework.Audio;

namespace MSFTTranslator
{
    public partial class MainPage : PhoneApplicationPage
    {
        public string FromLanguage { get; set; }
        const string privateTempFile = "TMPTRANSLATED.wav"; 
        public string ToLanguage { get; set; }

        private IsolatedStorageFile m_UserStore = null;
        private IsolatedStorageFile UserStore
        {
            get 
            {
                if (m_UserStore == null)
                {
                    m_UserStore = IsolatedStorageFile.GetUserStoreForApplication();
                }
                return m_UserStore;
            }
        }

        private OutMsg outmsg = new OutMsg();

        // Constructor
        public MainPage()
        {
            InitializeComponent();
            this.Loaded += new RoutedEventHandler(MainPage_Loaded);
            mediaElement.MediaFailed += new EventHandler<ExceptionRoutedEventArgs>(mediaElement_MediaFailed);
            FromLanguage = "en";
            ToLanguage = "es";
        }

        void mediaElement_MediaFailed(object sender, ExceptionRoutedEventArgs e)
        {
            MessageBox.Show("MediaFailed: " + e.ErrorException.ToString());
            outmsg.WriteLine("MediaFailed: " + e.ErrorException.ToString());
        }

        void MainPage_Loaded(object sender, RoutedEventArgs e)
        {
            // LoadLanguages();
        }

        private string m_AppId = String.Empty;
        private string AppId 
        {
            get
            {
                if (m_AppId == string.Empty)
                {
                    m_AppId = Application.Current.Resources["BingAPIAppId"].ToString();
                    if ((m_AppId == null) || (m_AppId.Length <= 0)) throw new Exception("LoadLanguages: invalid appid");
                }
                return m_AppId;
            }
        }


        private void LoadLanguages()
        {
            string appId = AppId;

            string uri = "http://api.microsofttranslator.com/v2/Http.svc/GetLanguagesForTranslate?appId=" + appId;
            
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(uri);
            request.BeginGetResponse(new AsyncCallback(ReadLanguagesWebRequestCallback), request);             
        }


        private void ReadLanguagesWebRequestCallback(IAsyncResult callbackResult )
        {
            outmsg.WriteLine("ReadLanguagesWebRequestCallback");
            // msdn
            HttpWebRequest myRequest = (HttpWebRequest)callbackResult.AsyncState;
            WebResponse myResponse = (HttpWebResponse)myRequest.EndGetResponse(callbackResult);
            
            using ( Stream stream = myResponse.GetResponseStream() )
            {
                System.Runtime.Serialization.DataContractSerializer dcs = new System.Runtime.Serialization.DataContractSerializer(typeof(List<string>));

                List<string> results = (List<string>)dcs.ReadObject(stream);
                // string languages = ("The following languages are available for translation: <br />");

                foreach (string language in results)
                {
                    System.Diagnostics.Debug.WriteLine(language);
                }
            }

        }

private void buttonTranslate_Click(object sender, RoutedEventArgs e)
{
    DoTranslate();
}

private void DoTranslate()
{
    outmsg.WriteLine("dotranslate");
    string appId = AppId;

    string uri = String.Format("http://api.microsofttranslator.com/v2/Http.svc/Translate?appId={0}&from={1}&to={2}&text={3}", appId, this.FromLanguage, this.ToLanguage, textBoxFrom.Text);

    HttpWebRequest request = (HttpWebRequest)WebRequest.Create(uri);
    request.BeginGetResponse(new AsyncCallback(ReadTranslateWebRequestCallback), request);
}

private void ReadTranslateWebRequestCallback(IAsyncResult callbackResult)
{
    outmsg.WriteLine("ReadTranslateWebRequestCallback");
    // msdn
    HttpWebRequest myRequest = (HttpWebRequest)callbackResult.AsyncState;
    WebResponse myResponse = (HttpWebResponse)myRequest.EndGetResponse(callbackResult);

    string translation = string.Empty;
    using (Stream stream = myResponse.GetResponseStream())
    {
        System.Runtime.Serialization.DataContractSerializer dcs = new System.Runtime.Serialization.DataContractSerializer(translation.GetType());
        translation = (string)dcs.ReadObject(stream);
    }

    Dispatcher.BeginInvoke(() =>
        textBoxTo.Text = translation
        );
}

        private void buttonSpeak_Click(object sender, RoutedEventArgs e)
        {
            DoSpeak();
        }


    //string appId = "myAppId";
    //string text = "Speak this for me";
    //string language = "en";

    //string uri = "http://api.microsofttranslator.com/v2/Http.svc/Speak?appId=" + appId +
    //        "&text=" + text + "&language=" + language;
    //HttpWebRequest httpWebRequest = (HttpWebRequest)WebRequest.Create(uri);
    //WebResponse resp = httpWebRequest.GetResponse();
    //Stream strm = resp.GetResponseStream();

    //SoundPlayer player = new SoundPlayer(strm);
    //player.Play();

        private void DoSpeak()
        {
            if ((textBoxTo.Text == "?")||(textBoxTo.Text==string.Empty) )
            {
                Dispatcher.BeginInvoke( () =>
                    MessageBox.Show("must do a translation first!")
                );
                return;
            }
            System.Diagnostics.Debug.WriteLine("DoSpeak");

            string appId = AppId;
            string uri = String.Format("http://api.microsofttranslator.com/v2/Http.svc/Speak?appId={0}&text={1}&language={2}", appId, textBoxTo.Text, this.ToLanguage);
            outmsg.WriteLine("uri=" + uri);
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(uri);
            request.BeginGetResponse(new AsyncCallback(ReadSpeakWebRequestCallback), request);  
        }

        private void ReadSpeakWebRequestCallback(IAsyncResult callbackResult)
        {
            outmsg.WriteLine("ReadSpeakWebWebRequestCallback");
 

            HttpWebRequest myRequest = (HttpWebRequest)callbackResult.AsyncState;
            WebResponse myResponse = (HttpWebResponse)myRequest.EndGetResponse(callbackResult);

            Microsoft.Xna.Framework.FrameworkDispatcher.Update();
            SoundEffect se = SoundEffect.FromStream(myResponse.GetResponseStream());
            se.Play();
        }

    }
}