﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;


namespace SAMSMobileCitizenry
{
    public class SystemInfoHelper
    {
        public string GetAppVersion()
        {
            System.Reflection.Assembly assembly = System.Reflection.Assembly.GetExecutingAssembly();
            var name = new System.Reflection.AssemblyName(assembly.FullName);
            return name.Version.ToString();
        }

        public string GetErrorInfo()
        {
            System.Text.StringBuilder sb = new System.Text.StringBuilder(2048);
            sb.AppendLine("MobileCitizenry Unhandled Error Report");
            sb.AppendLine(System.DateTime.Now.ToString());
            sb.AppendLine(String.Format("AppVersion: {0}", GetAppVersion()));
            sb.AppendLine(DeviceExtendedPropertiesLongHelper("ApplicationCurrentMemoryUsage"));
            sb.AppendLine(DeviceExtendedPropertiesLongHelper("ApplicationPeakMemoryUsage"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceFirmwareVersion"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceHardwareVersion"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceManufacturer"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceName"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceManufacturer"));
            sb.AppendLine(DeviceExtendedPropertiesStringHelper("DeviceTotalMemory"));
            sb.AppendLine(String.Format("DeviceUniqueId: {0}", DeviceUniqueId()));
            sb.AppendLine(String.Format("Culture: {0}", System.Globalization.CultureInfo.CurrentCulture));

            sb.AppendLine(String.Format("OS  Version: {0}", Environment.OSVersion));
            sb.AppendLine(String.Format("CLR Version: {0}", Environment.Version));
            sb.AppendLine(String.Format("Device Type: {0}", Microsoft.Devices.Environment.DeviceType));
            sb.AppendLine(String.Format("Network Type: {0}", Microsoft.Phone.Net.NetworkInformation.NetworkInterface.NetworkInterfaceType));

            using (var appStorage = System.IO.IsolatedStorage.IsolatedStorageFile.GetUserStoreForApplication())
            {
                sb.Append(String.Format("Iso Storage AvailableFreeSpace: {0:f3} MB", appStorage.AvailableFreeSpace / 1024f / 1024f));
            }

            return sb.ToString();
        }

        private string DeviceExtendedPropertiesStringHelper(string p)
        {
            string retval = String.Empty;
            try
            {
                object _prop;
                Microsoft.Phone.Info.DeviceExtendedProperties.TryGetValue(p, out _prop);
                string _val = (string)_prop;
                retval = String.Format("{0}: {1}", p, _val.ToString());
            }
            catch (Exception ex)
            {
                retval = "error";
            }
            return retval;
        }

        public string DeviceExtendedPropertiesLongHelper(string p)
        {
            string retval = String.Empty;
            try
            {
                object _prop;
                Microsoft.Phone.Info.DeviceExtendedProperties.TryGetValue(p, out _prop);
                long _val = (long)_prop;
                retval = String.Format("{0}: {1}", p, _val.ToString());
            }
            catch (Exception ex)
            {
                retval = "error";
            }
            return retval;
        }

        public string DeviceUniqueId()
        {
            System.Text.StringBuilder _retval = new System.Text.StringBuilder(1024);
            object _prop;

            Microsoft.Phone.Info.DeviceExtendedProperties.TryGetValue("DeviceUniqueId", out _prop);

            if (_prop == null)
            {
                _retval.Append("Set ID_CAP_IDENTITY_DEVICE");
            }
            else
            {
                // DeviceUniqueId returns a byte array
                foreach (byte b in (byte[])_prop)
                    _retval.Append((char)(b % 64 + 32));
            }
            return _retval.ToString();
        } // DeviceUniqueId
    }
}
